import pandas as pd
from mlxtend.frequent_patterns import apriori, association_rules


df=pd.read_stata("bachlor_dataset regresjonsvariabler.dta")
df1=df[["read", "sovjet", "teacher_age", "teacher_cert", "sameteacher1_less"]]


def bucketizingdata(df): #Changing values of read to number 1-4 depending of which 25% they relate to.
    newdf = df.copy()
    for col in newdf:
        if col not in ["teacher_age", "teacher_cert","sameteacher1_less", "sovjet"]:
            newdf[col] = pd.qcut(newdf[col], 4, labels=['1', '2', '3', '4'])
    newdf=newdf[newdf.read == '4']
    print(newdf)
    return newdf


def dummifyandapriori(df): #Dummifies all values used and uses the apriori algorithm to generate a list of frequent itemsets, which are then filtered by parameters of choice
    dummylist = []
    for att in df:
        if att in ["read", "teacher_age", "teacher_cert","sameteacher1_less", "sovjet"]:
            df[att] = df[att].astype("category")
        dummylist.append(pd.get_dummies(df[[att]]))
    dummified_df = pd.concat(dummylist, axis=1)

    frequent_itemsets = {}
    minpaterns = 300
    minsup = 1.0
    minconf = 0.9
    while minsup > 0:
        minsup = minsup * 0.95
        frequent_itemsets = apriori(dummified_df, min_support=minsup, use_colnames=True)
        if len(frequent_itemsets) >= minpaterns:
            print("The perfect teacher:")
            print("Minimum support:", minsup)
            break
    print("Number of patterns:", len(frequent_itemsets))
    rules = association_rules(frequent_itemsets, metric="confidence", min_threshold=minconf)
    rules["antecedent_len"] = rules["antecedents"].apply(lambda x: len(x))
    rules["consequents_len"] = rules["consequents"].apply(lambda x: len(x))
    newrules = rules[(rules['antecedent_len'] >= 1)]
    newrules = newrules[(rules['consequents_len'] == 1)]
    #newrules = newrules[(newrules['consequents']=='read_10')]
    if len(newrules) < 1:
        print("No rules found with given threshold")
    sortednewrules = newrules.sort_values("lift", ascending=False)
    score_lift = mean_lift(sortednewrules)
    score_confidence = mean_confidence(sortednewrules)
    print("Mean lift:", score_lift)
    print("Mean confidence:", score_confidence)
    return sortednewrules


def top_rules(rules): #selects the 100 top rules
    top_df=rules.head(100)
    return top_df


def patternmining_teacher(df):
    a= dummifyandapriori(bucketizingdata(df))
    return a


def main(): #Connects all the tasks and saves the file as a CSV
    top_rules(patternmining_teacher(df1)).to_csv('YOURFILENAME.csv',index=True,header=True)


if __name__=="__main__":
    main()


def mean_lift(result):
    metric = "lift"
    measure = result.loc[:, metric]
    measure_sum = 0
    for m in measure:
        measure_sum = measure_sum + m
    if len(measure) == 0:
        mean_measure = 0
    else:
        mean_measure = measure_sum / len(measure)
    return mean_measure


def mean_confidence(result):
    metric = "confidence"
    measure = result.loc[:, metric]
    measure_sum = 0
    for m in measure:
        measure_sum = measure_sum + m
    if len(measure) == 0:
        mean_measure = 0
    else:
        mean_measure = measure_sum / len(measure)
    return mean_measure
